function POLY = inscribePlatonicSolid(shape,r)
% function POLY = inscribePlatonicSolid(shape,r)
% Regular Polyhedron Generators 
% 
% Regular Polyhedron are three dimensional objects that have
% one type of regular polygon on the surface. There are only
% a few:
%        name        faces      face type           vertices  edges
%     Tetrahedron       4   equilateral triangles      4        6
%     Cube              6   squares                    8       12
%     Dodecahedron     12   pentagons                 20       30
%     Octahedron        8   equilateral triangles      6       12
%     Icosahedron      12   equilateral triangles     12       30
%     Sphere          inf   technically not a regular polyhedron
% This is code based on a pseudocode at:
% Pseudocode assumes all variables of type double except for i, j, k
% which are obviously integer. The code was extracted from OpenGL
% test programs similar to the commonly available "cube.c". This is
% original code based on mathematics, not copied, covered by GNU
% General Public License. 
% http://www.csee.umbc.edu/~squire/reference/polyhedra.shtml
%
% (C)opyright 2005, Bennett Landman, bennett@bme.jhu.edu
% Revision History:
% Created: 2/16/2005

switch(lower(shape))
    case 'tetrahedron'
        POLY = tetrahedron(r);
    case 'cube'
        POLY = cube(r);
    case 'dodecahedron'
        POLY = dodecahedron(r);
    case 'octahedron'
        POLY = octahedron(r);
    case 'icosahedron'
        POLY = icosahedron(r);
    otherwise
        warning('Unknown platonic solid');
end

%% TETRAHEDRON %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function POLY = tetrahedron(r)

phiaa  = -19.471220333; % /* the phi angle needed for generation */


phia = pi*phiaa/180.0; % /* 1 set of three points */
the120 = pi*120.0/180.0;
vertices(1,:) =  [0 0 r];
the = 0.0;
for i=1:3

    vertices(i+1,:) = [r*cos(the)*cos(phia) r*sin(the)*cos(phia) r*sin(phia)];
    the = the+the120;
end
%/* map vertices to 4 faces */
POLY.vert = vertices;
POLY.face{1} = [1 2 3];
POLY.face{2} = [1 3 4];
POLY.face{3} = [1 4 2];
POLY.face{4} = [2 3 4];

%% CUBE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function POLY = cube(r)

phiaa = 35.264391; % /* the phi needed for generation */


phia = pi*phiaa/180.0; %/* 2 sets of four points */
phib = -phia;
the90 = pi*90.0/180.0;
the = 0.0;
for i=0:3

    vertices(i+1,:) = [r*cos(the)*cos(phia) r*sin(the)*cos(phia) r*sin(phia)];
    the = the+the90;
end
the=0.0;
for i=4:7

    vertices(i+1,:)=[r*cos(the)*cos(phib) r*sin(the)*cos(phib) r*sin(phib)];
    the = the+the90;
end

% /* map vertices to 6 faces */
POLY.vert = vertices;
POLY.face{1} = [0,1,2,3]+1;
POLY.face{2} = [4,5,6,7]+1;
POLY.face{3} = [0,1,5,4]+1;
POLY.face{4} = [1,2,6,5]+1;
POLY.face{5} = [2,3,7,6]+1;
POLY.face{6} = [3,0,4,7]+1;

%% DODECAHEDRON %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function POLY = dodecahedron(r)

   phiaa = 52.62263590; % /* the two phi angles needed for generation */
   phibb = 10.81231754;

  
  phia = pi*phiaa/180.0; % /* 4 sets of five points each */
  phib = pi*phibb/180.0;
  phic = pi*(-phibb)/180.0;
  phid = pi*(-phiaa)/180.0;
  the72 = pi*72.0/180;
  theb = the72/2.0; % /* pairs of layers offset 36 degrees */
  the = 0.0;
  for i=0:4
  
    vertices(i+1,:) = [r*cos(the)*cos(phia) r*sin(the)*cos(phia) r*sin(phia)];
    the = the+the72;
    end
  the=0.0;
  for i=5:9
  
    vertices(i+1,:) = [r*cos(the)*cos(phib) r*sin(the)*cos(phib) r*sin(phib)];
    the = the+the72;
  end
  the = theb;
  for i=10:14 
  
    vertices(i+1,:) = [r*cos(the)*cos(phic) r*sin(the)*cos(phic) r*sin(phic)];
    the = the+the72;
  end
  the=theb;
  for i=15:19
      
    vertices(i+1,:) =[ r*cos(the)*cos(phid) r*sin(the)*cos(phid) r*sin(phid)];
    the = the+the72;
  end

  %/* map vertices to 12 faces */
  POLY.vert = vertices;
POLY.face{1} = [0,1,2,3,4]+1;
POLY.face{2} = [0,1,6,10,5]+1;
POLY.face{3} = [1,2,7,11,6]+1;
POLY.face{4} = [2,3,8,12,7]+1;
POLY.face{5} = [3,4,9,13,8]+1;
POLY.face{6} = [4,0,5,14,9]+1;
POLY.face{7} = [15,16,11,6,10]+1;
POLY.face{8} = [16,17,12,7,11]+1;
POLY.face{9} = [17,18,13,8,12]+1;
POLY.face{10} = [18,19,14,9,13]+1;
POLY.face{11} = [19,15,10,5,14]+1;
POLY.face{12} = [15,16,17,18,19]+1;
  
%% OCTAHEDRON %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function POLY = octahedron(r)

   phiaa  = 0.0; % /* the phi needed for generation */
  
  phia = pi*phiaa/180.0; % /* 1 set of four points */
  the90 = pi*90.0/180;
  vertices(1,:) = [0 0 r];

  vertices(6,:) = [0 0 -r]; 
  the = 0.0;
  for i=1:4
  
    vertices(i+1,:) =[r*cos(the)*cos(phia) r*sin(the)*cos(phia) r*sin(phia)];
    the = the+the90;
  end

  %/* map vertices to 8 faces */
    POLY.vert = vertices;
POLY.face{1} = [0,1,2]+1;
POLY.face{2} = [0,2,3]+1;
POLY.face{3} = [0,3,4]+1;
POLY.face{4} = [0,4,1]+1;
POLY.face{5} = [5,1,2]+1;
POLY.face{6} = [5,2,3]+1;
POLY.face{7} = [5,3,4]+1;
POLY.face{8} = [5,4,1]+1;

%% ICOSAHEDRON %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function POLY = icosahedron(r)

  phiaa  = 26.56505; %/* phi needed for generation */
  phia = pi*phiaa/180.0; %/* 2 sets of four points */
  theb = pi*36.0/180.0;  %/* offset second set 36 degrees */
  the72 = pi*72.0/180;  % /* step 72 degrees */
  vertices(1,:) = [0 0 r];
  vertices(12,:) = [0 0 -r];
  the = 0.0;
  for i=1:5
  
    vertices(i+1,:) = [r*cos(the)*cos(phia) r*sin(the)*cos(phia) r*sin(phia)];
    the = the+the72;
  end
  the=theb;
  for i=6:10
  
    vertices(i+1,:) = [r*cos(the)*cos(-phia) r*sin(the)*cos(-phia) r*sin(-phia)];
    the = the+the72;
  end

  % /* map vertices to 20 faces */
    POLY.vert = vertices;
POLY.face{1} = [0,1,2]+1;
POLY.face{2} = [0,2,3]+1;
POLY.face{3} = [0,3,4]+1;
POLY.face{4} = [0,4,5]+1;
POLY.face{5} = [0,5,1]+1;
POLY.face{6} = [11,6,7]+1;
POLY.face{7} = [11,7,8]+1;
POLY.face{8} = [11,8,9]+1;
POLY.face{9} = [11,9,10]+1;
POLY.face{10} = [11,10,6]+1;
POLY.face{11} = [1,2,6]+1;
POLY.face{12} = [2,3,7]+1;
POLY.face{13} = [3,4,8]+1;
POLY.face{14} = [4,5,9]+1;
POLY.face{15} = [5,1,10]+1;
POLY.face{16} = [6,7,2]+1;
POLY.face{17} = [7,8,3]+1;
POLY.face{18} = [8,9,4]+1;
POLY.face{19} = [9,10,5]+1;
POLY.face{20} = [10,6,1]+1;



